@@ -40,6 +40,7 @@ module Data.IntPSQ.Internal
     , insertView
     , deleteView
     , minView
+    , atMostView
 
       -- * Traversal
     , map
@@ -63,21 +64,17 @@ module Data.IntPSQ.Internal
 
 import           Control.Applicative ((<$>), (<*>))
 import           Control.DeepSeq     (NFData (rnf))
-
 import           Data.Bits
 import           Data.BitUtil
-import           Data.Foldable       (Foldable (foldr))
+import           Data.Foldable       (Foldable)
 import           Data.List           (foldl')
-import           Data.Maybe          (isJust)
-import           Data.Word           (Word)
-
 import qualified Data.List           as List
-
+import           Data.Maybe          (isJust)
+import           Data.Traversable
+import           Data.Word           (Word)
 import           Prelude             hiding (filter, foldl, foldr, lookup, map,
                                       null)
 
-import           Data.Traversable
-
 -- TODO (SM): get rid of bang patterns
 
 {-
@@ -197,9 +194,9 @@ lookup k = go
 -- | /O(1)/ The element with the lowest priority.
 findMin :: Ord p => IntPSQ p v -> Maybe (Int, p, v)
 findMin t = case t of
-    Nil              -> Nothing
-    Tip k p x        -> Just (k, p, x)
-    Bin k p x _ _ _  -> Just (k, p, x)
+    Nil             -> Nothing
+    Tip k p x       -> Just (k, p, x)
+    Bin k p x _ _ _ -> Just (k, p, x)
 
 
 ------------------------------------------------------------------------------
@@ -370,7 +367,7 @@ toList :: IntPSQ p v -> [(Int, p, v)]
 toList =
     go []
   where
-    go acc Nil                = acc
+    go acc Nil                   = acc
     go acc (Tip k' p' x')        = (k', p', x') : acc
     go acc (Bin k' p' x' _m l r) = (k', p', x') : go (go acc r) l
 
@@ -431,6 +428,26 @@ minView t = case t of
     Tip k p x       -> Just (k, p, x, Nil)
     Bin k p x m l r -> Just (k, p, x, merge m l r)
 
+-- | Return a list of elements ordered by key whose priorities are at most @pt@,
+-- and the rest of the queue stripped of these elements.  The returned list of
+-- elements can be in any order: no guarantees there.
+{-# INLINABLE atMostView #-}
+atMostView :: Ord p => p -> IntPSQ p v -> ([(Int, p, v)], IntPSQ p v)
+atMostView pt t0 = go [] t0
+  where
+    go acc t = case t of
+        Nil             -> (acc, t)
+        Tip k p x
+            | p > pt    -> (acc, t)
+            | otherwise -> ((k, p, x) : acc, Nil)
+
+        Bin k p x m l r
+            | p > pt    -> (acc, t)
+            | otherwise ->
+                let (acc',  l') = go acc  l
+                    (acc'', r') = go acc' r
+                in  ((k, p, x) : acc'', merge m l' r')
+
 
 ------------------------------------------------------------------------------
 -- Traversal
@@ -696,6 +713,6 @@ validMask (Bin _ _ _ m left right ) =
         Just xoredKeys ->
             fromIntegral mask == highestBitMask (fromIntegral xoredKeys)
 
-    childKey Nil = Nothing
-    childKey (Tip k _ _) = Just k
+    childKey Nil               = Nothing
+    childKey (Tip k _ _)       = Just k
     childKey (Bin k _ _ _ _ _) = Just k
